@@ -1,5 +1,5 @@
 /* Openoffice.org-voikko: Finnish linguistic extension for OpenOffice.org
- * Copyright (C) 2007 - 2008 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2007 - 2009 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,17 +29,30 @@ namespace voikko {
 PropertyManager::PropertyManager(uno::Reference<uno::XComponentContext> cContext):
 	compContext(cContext),
 	linguEventListeners(getVoikkoMutex()) {
-	VOIKKO_DEBUG("PropertyManager created");
+	VOIKKO_DEBUG("PropertyManager:CTOR");
 	isInitialized = sal_False;
 	linguPropSet = 0;
 	hyphMinLeading = 2;
 	hyphMinTrailing = 2;
 	hyphMinWordLength = 5;
 	hyphWordParts = sal_False;
+	hyphUnknownWords = sal_True;
+	try {
+		uno::Any dictVariantA = readFromRegistry(
+		              A2OU("/org.puimula.ooovoikko.Config/dictionary"),
+		              A2OU("variant"));
+		dictVariantA >>= dictVariant;
+		VOIKKO_DEBUG_2("Initial dictionary variant '%s'", OU2DEBUG(dictVariant));
+	}
+	catch (beans::UnknownPropertyException e) {
+		VOIKKO_DEBUG("Setting initial dictionary variant to default");
+		dictVariant = A2OU("standard");
+	}
+	initialize();
 }
 
 PropertyManager::~PropertyManager() {
-	VOIKKO_DEBUG("PropertyManager: running destructor");
+	VOIKKO_DEBUG("PropertyManager:DTOR");
 	/* This might need locking, but since the property manager is never destroyed
 	 * before the office shutdown, there should be no other sources for calls to
 	 * libvoikko at this time. */
@@ -49,38 +62,77 @@ PropertyManager::~PropertyManager() {
 	}
 }
 
-void SAL_CALL PropertyManager::propertyChange(const beans::PropertyChangeEvent &)
+void SAL_CALL PropertyManager::propertyChange(const beans::PropertyChangeEvent & pce)
 	throw (uno::RuntimeException) {
-	// FIXME: property change notifications are not being received
-	// VOIKKO_DEBUG_2("PropertyManager::propertyChange: %s", OU2DEBUG(pce.PropertyName));
+	VOIKKO_DEBUG_2("PropertyManager::propertyChange: %s", OU2DEBUG(pce.PropertyName));
+	setProperties(linguPropSet);
+	linguistic2::LinguServiceEvent event;
+	event.nEvent =  linguistic2::LinguServiceEventFlags::SPELL_CORRECT_WORDS_AGAIN;
+	event.nEvent |= linguistic2::LinguServiceEventFlags::SPELL_WRONG_WORDS_AGAIN;
+	event.nEvent |= linguistic2::LinguServiceEventFlags::HYPHENATE_AGAIN;
+	event.nEvent |= linguistic2::LinguServiceEventFlags::PROOFREAD_AGAIN;
+	sendLinguEvent(event);
 }
 
 void SAL_CALL PropertyManager::disposing(const lang::EventObject &)
 	throw (uno::RuntimeException){
-	VOIKKO_DEBUG("PropertyManager::disposing");
+}
+
+void PropertyManager::initLibvoikko() {
+	if (voikko_initialized) {
+		voikko_terminate(voikko_handle);
+		voikko_initialized = sal_False;
+	}
+	
+	OString variantAscii = OUStringToOString(dictVariant, RTL_TEXTENCODING_UTF8);
+	
+	const char * error = initLibvoikkoWithVariant(variantAscii.getStr());
+	if (error) {
+		VOIKKO_DEBUG_2("Failed to initialize voikko with specific variant: %s", error);
+		VOIKKO_DEBUG("Trying next with default variant");
+		voikkoErrorString = initLibvoikkoWithVariant("default");
+		if (voikkoErrorString) {
+			VOIKKO_DEBUG_2("ERROR: Failed to initialize voikko with default variant: %s", voikkoErrorString);
+			return;
+		}
+	}
+	else {
+		voikkoErrorString = 0;
+	}
+	
+	voikko_set_bool_option(voikko_handle, VOIKKO_OPT_IGNORE_DOT, 1);
+	voikko_set_bool_option(voikko_handle, VOIKKO_OPT_NO_UGLY_HYPHENATION, 1);
+	
+	// Set these options globally until OOo bug #97945 is resolved.
+	voikko_set_bool_option(voikko_handle, VOIKKO_OPT_ACCEPT_TITLES_IN_GC, 1);
+	voikko_set_bool_option(voikko_handle, VOIKKO_OPT_ACCEPT_BULLETED_LISTS_IN_GC, 1);
+	
+	voikko_set_bool_option(voikko_handle, VOIKKO_OPT_ACCEPT_UNFINISHED_PARAGRAPHS_IN_GC, 1);
+	voikko_initialized = sal_True;
+	VOIKKO_DEBUG("PropertyManager::initLibvoikko: libvoikko initalized");
+}
+
+const char * PropertyManager::initLibvoikkoWithVariant(const char * variant) {
+	#ifdef VOIKKO_STANDALONE_EXTENSION
+		rtl_TextEncoding encoding = osl_getTextEncodingFromLocale(0);
+		if (encoding == RTL_TEXTENCODING_DONTKNOW) {
+			encoding = RTL_TEXTENCODING_UTF8;
+		}
+		return voikko_init_with_path(&voikko_handle, variant, 0,
+		       OUStringToOString(getInstallationPath(compContext), encoding).getStr());
+	#else
+		return voikko_init(&voikko_handle, variant, 0);
+	#endif
 }
 
 void PropertyManager::initialize() throw (uno::Exception) {
 	VOIKKO_DEBUG("PropertyManager::initialize: starting");
 	if (!voikko_initialized) {
 		isInitialized = sal_False;
-		#ifdef VOIKKO_STANDALONE_EXTENSION
-			rtl_TextEncoding encoding = osl_getTextEncodingFromLocale(0);
-			if (encoding == RTL_TEXTENCODING_DONTKNOW)
-				encoding = RTL_TEXTENCODING_UTF8;
-			voikkoErrorString = voikko_init_with_path(&voikko_handle, "fi_FI", 0,
-				OUStringToOString(getInstallationPath(compContext), encoding).getStr());
-		#else
-			voikkoErrorString = voikko_init(&voikko_handle, "fi_FI", 0);
-		#endif
-		if (voikkoErrorString) {
-			VOIKKO_DEBUG_2("Failed to initialize voikko: %s", voikkoErrorString);
+		initLibvoikko();
+		if (!voikko_initialized) {
 			return;
 		}
-		voikko_set_string_option(voikko_handle, VOIKKO_OPT_ENCODING, "UTF-8");
-		voikko_set_bool_option(voikko_handle, VOIKKO_OPT_IGNORE_DOT, 1);
-		voikko_set_bool_option(voikko_handle, VOIKKO_OPT_NO_UGLY_HYPHENATION, 1);
-		voikko_set_bool_option(voikko_handle, VOIKKO_OPT_ACCEPT_TITLES_IN_GC, 1);
 		
 		// Determine UI language
 		messageLanguage = "en_US";
@@ -104,10 +156,9 @@ void PropertyManager::initialize() throw (uno::Exception) {
 					messageLanguage = "fi_FI";
 			}
 		}
-		catch (beans::UnknownPropertyException) { }
-		
-		voikko_initialized = sal_True;
-		VOIKKO_DEBUG("PropertyManager::initialize: libvoikko initalized");
+		catch (beans::UnknownPropertyException) {
+			VOIKKO_DEBUG("ERROR: PropertyManager::initialize caught UnknownPropertyException");
+		}
 	}
 	if (!isInitialized) {
 		uno::Reference<lang::XMultiComponentFactory> servManager =
@@ -130,6 +181,7 @@ void PropertyManager::initialize() throw (uno::Exception) {
 	event.nEvent =  linguistic2::LinguServiceEventFlags::SPELL_CORRECT_WORDS_AGAIN;
 	event.nEvent |= linguistic2::LinguServiceEventFlags::SPELL_WRONG_WORDS_AGAIN;
 	event.nEvent |= linguistic2::LinguServiceEventFlags::HYPHENATE_AGAIN;
+	event.nEvent |= linguistic2::LinguServiceEventFlags::PROOFREAD_AGAIN;
 	sendLinguEvent(event);
 }
 
@@ -143,7 +195,7 @@ sal_Bool PropertyManager::addLinguServiceEventListener(
 	throw (uno::RuntimeException) {
 	VOIKKO_DEBUG("PropertyManager::addLinguServiceEventListener");
 	if (xLstnr.is()) {
-		sal_Int32	listenerCount = linguEventListeners.getLength();
+		sal_Int32 listenerCount = linguEventListeners.getLength();
 		return (linguEventListeners.addInterface(xLstnr) != listenerCount);
 	}
 	else return sal_False;
@@ -154,7 +206,7 @@ sal_Bool PropertyManager::removeLinguServiceEventListener(
 	throw (uno::RuntimeException) {
 	VOIKKO_DEBUG("PropertyManager::removeLinguServiceEventListener");
 	if (xLstnr.is()) {
-		sal_Int32	listenerCount = linguEventListeners.getLength();
+		sal_Int32 listenerCount = linguEventListeners.getLength();
 		return (linguEventListeners.removeInterface(xLstnr) != listenerCount);
 	}
 	else return sal_False;
@@ -184,6 +236,11 @@ void PropertyManager::readVoikkoSettings() {
 			A2OU("/org.puimula.ooovoikko.Config/hyphenator"),
 			A2OU("hyphWordParts"));
 		hyphWordParts >>= this->hyphWordParts;
+		
+		uno::Any hyphUnknownWords = readFromRegistry(
+			A2OU("/org.puimula.ooovoikko.Config/hyphenator"),
+			A2OU("hyphUnknownWords"));
+		hyphUnknownWords >>= this->hyphUnknownWords;
 	}
 	catch (beans::UnknownPropertyException e) {
 		VOIKKO_DEBUG("ERROR: readVoikkoSettings: UnknownPropertyException");
@@ -200,6 +257,50 @@ const char * PropertyManager::getMessageLanguage() {
 	return messageLanguage;
 }
 
+void PropertyManager::reloadVoikkoSettings() {
+	linguistic2::LinguServiceEvent event;
+	event.nEvent = 0;
+	try {
+		uno::Any hyphWordParts = readFromRegistry(
+			A2OU("/org.puimula.ooovoikko.Config/hyphenator"),
+			A2OU("hyphWordParts"));
+		sal_Bool hyphWordPartsNew = this->hyphWordParts;
+		hyphWordParts >>= hyphWordPartsNew;
+		if (hyphWordPartsNew != this->hyphWordParts) {
+			event.nEvent |= linguistic2::LinguServiceEventFlags::HYPHENATE_AGAIN;
+			this->hyphWordParts = hyphWordPartsNew;
+		}
+		
+		uno::Any hyphUnknownWords = readFromRegistry(
+			A2OU("/org.puimula.ooovoikko.Config/hyphenator"),
+			A2OU("hyphUnknownWords"));
+		sal_Bool hyphUnknownWordsNew = this->hyphUnknownWords;
+		hyphUnknownWords >>= hyphUnknownWordsNew;
+		if (hyphUnknownWordsNew != this->hyphUnknownWords) {
+			event.nEvent |= linguistic2::LinguServiceEventFlags::HYPHENATE_AGAIN;
+			this->hyphUnknownWords = hyphUnknownWordsNew;
+		}
+		
+		uno::Any dictVariantA = readFromRegistry(
+			A2OU("/org.puimula.ooovoikko.Config/dictionary"),
+			A2OU("variant"));
+		OUString dictVariantNew = this->dictVariant;
+		dictVariantA >>= dictVariantNew;
+		if (dictVariantNew != this->dictVariant) {
+			event.nEvent |= linguistic2::LinguServiceEventFlags::SPELL_CORRECT_WORDS_AGAIN;
+			event.nEvent |= linguistic2::LinguServiceEventFlags::SPELL_WRONG_WORDS_AGAIN;
+			event.nEvent |= linguistic2::LinguServiceEventFlags::PROOFREAD_AGAIN;
+			this->dictVariant = dictVariantNew;
+			initLibvoikko();
+		}
+	}
+	catch (beans::UnknownPropertyException e) {
+		VOIKKO_DEBUG("ERROR: PropertyManager::reloadVoikkoSettings: UnknownPropertyException");
+	}
+	syncHyphenatorSettings();
+	sendLinguEvent(event);
+}
+
 void PropertyManager::setProperties(const uno::Reference<beans::XPropertySet> & properties) {
 	beans::PropertyValue pValue;
 	uno::Sequence<beans::Property> pSeq = properties->getPropertySetInfo()->getProperties();
@@ -270,21 +371,40 @@ void PropertyManager::setValue(const beans::PropertyValue & value) {
 	}
 }
 
-inline void PropertyManager::syncHyphenatorSettings() {
-	if (hyphWordParts)
+void PropertyManager::syncHyphenatorSettings() {
+	if (hyphWordParts) {
 		voikko_set_int_option(voikko_handle, VOIKKO_MIN_HYPHENATED_WORD_LENGTH,
 		                      hyphMinWordLength);
-	else
+	}
+	else {
 		voikko_set_int_option(voikko_handle, VOIKKO_MIN_HYPHENATED_WORD_LENGTH, 2);
+	}
+	
+	voikko_set_bool_option(voikko_handle, VOIKKO_OPT_HYPHENATE_UNKNOWN_WORDS,
+	                       hyphUnknownWords ? 1 : 0);
 }
 
 void PropertyManager::sendLinguEvent(const linguistic2::LinguServiceEvent & event) {
+	VOIKKO_DEBUG("PropertyManager::sendLinguEvent");
 	cppu::OInterfaceIteratorHelper iterator(linguEventListeners);
 	while (iterator.hasMoreElements()) {
 		uno::Reference<linguistic2::XLinguServiceEventListener>
 			ref(iterator.next(), uno::UNO_QUERY);
-		if (ref.is()) ref->processLinguServiceEvent(event);
+		if (ref.is()) {
+			VOIKKO_DEBUG("PropertyManager::sendLinguEvent sending event");
+			ref->processLinguServiceEvent(event);
+		}
 	}
 }
 
+static uno::Reference<voikko::PropertyManager> thePropertyManager;
+
+uno::Reference<voikko::PropertyManager> PropertyManager::get(uno::Reference<uno::XComponentContext> const & context) {
+	if (!thePropertyManager.is()) {
+		VOIKKO_DEBUG("PropertyManager::get first");
+		thePropertyManager = new PropertyManager(context);
+	}
+	return thePropertyManager;
+}
+
 }
